@@ -25,6 +25,11 @@ module Codec.Picture.Types( -- * Types
                           , thawImage
                           , unsafeThawImage
 
+                            -- ** Image Lenses
+                          , Traversal
+                          , imagePixels
+                          , imageIPixels
+
                             -- ** Pixel types
                           , Pixel8
                           , Pixel16
@@ -95,12 +100,13 @@ module Codec.Picture.Types( -- * Types
 
 #if !MIN_VERSION_base(4,8,0)
 import Data.Monoid( Monoid, mempty )
+import Control.Applicative( Applicative, pure, (<*>), (<$>) )
 #endif
 
 import Data.Monoid( (<>) )
 import Control.Monad( foldM, liftM, ap )
 import Control.DeepSeq( NFData( .. ) )
-import Control.Monad.ST( runST )
+import Control.Monad.ST( ST, runST )
 import Control.Monad.Primitive ( PrimMonad, PrimState )
 import Foreign.ForeignPtr( castForeignPtr )
 import Foreign.Storable ( Storable )
@@ -222,7 +228,7 @@ unsafeExtractComponent comp img@(Image { imageWidth = w, imageHeight = h })
   | comp >= padd = error $ "extractComponent : invalid component index ("
                          ++ show comp ++ ", max:" ++ show padd ++ ")"
   | otherwise = Image { imageWidth = w, imageHeight = h, imageData = plane }
-      where plane = stride img 1 padd comp
+      where plane = stride img padd comp
             padd = componentCount (undefined :: a)
 
 -- | For any image with an alpha component (transparency),
@@ -247,14 +253,6 @@ instance TransparentPixel PixelRGBA8 PixelRGB8 where
     {-# INLINE getTransparency #-}
     getTransparency (PixelRGBA8 _ _ _ a) = a
 
--- | Iteration from to n in monadic context, without data
--- keeping.
-lineMap :: (Monad m) => Int -> (Int -> m ()) -> m ()
-{-# INLINE lineMap #-}
-lineMap count f = go 0
-  where go n | n >= count = return ()
-        go n = f n >> go (n + 1)
-
 lineFold :: (Monad m) => a -> Int -> (a -> Int -> m a) -> m a
 {-# INLINE lineFold #-}
 lineFold initial count f = go 0 initial
@@ -262,19 +260,18 @@ lineFold initial count f = go 0 initial
         go n acc = f acc n >>= go (n + 1)
 
 stride :: (Storable (PixelBaseComponent a))
-       => Image a -> Int -> Int -> Int -> V.Vector (PixelBaseComponent a)
+       => Image a -> Int -> Int -> V.Vector (PixelBaseComponent a)
 stride Image { imageWidth = w, imageHeight = h, imageData = array }
-        run padd firstComponent = runST $ do
-    let cell_count = w * h * run
+        padd firstComponent = runST $ do
+    let cell_count = w * h
     outArray <- M.new cell_count
 
-    let strideWrite write_idx _ | write_idx == cell_count = return ()
-        strideWrite write_idx read_idx = do
-            lineMap (run - 1) $ \i ->
-                (outArray `M.unsafeWrite` (write_idx + i)) $ array `V.unsafeIndex` (read_idx + i)
-            strideWrite (write_idx + run) (read_idx + padd)
+    let go writeIndex _ | writeIndex >= cell_count = return ()
+        go writeIndex readIndex = do
+          (outArray `M.unsafeWrite` writeIndex) $ array `V.unsafeIndex` readIndex
+          go (writeIndex + 1) $ readIndex + padd
 
-    strideWrite 0 firstComponent
+    go 0 firstComponent
     V.unsafeFreeze outArray
 
 instance NFData (Image a) where
@@ -913,6 +910,81 @@ pixelMap f Image { imageWidth = w, imageHeight = h, imageData = vec } =
             -- outside of this where block
             V.unsafeFreeze newArr
 
+
+-- | Helpers to embed a rankNTypes inside an Applicative
+newtype GenST a = GenST { genAction :: forall s. ST s (M.STVector s a) }
+
+-- | Traversal type matching the definition in the Lens package.
+type Traversal s t a b =
+    forall f. Applicative f => (a -> f b) -> s -> f t 
+
+writePx :: Pixel px
+        => Int -> GenST (PixelBaseComponent px) -> px -> GenST (PixelBaseComponent px)
+{-# INLINE writePx #-}
+writePx idx act px = GenST $ do
+   vec <- genAction act
+   unsafeWritePixel vec idx px
+   return vec
+
+freezeGenST :: Pixel px
+            => Int -> Int -> GenST (PixelBaseComponent px) -> Image px
+freezeGenST w h act =
+  Image w h (runST (genAction act >>= V.unsafeFreeze))
+
+-- | Traversal in "raster" order, from left to right the top to bottom.
+-- This traversal is matching pixelMap in spirit.
+--
+-- Since 3.2.4
+imagePixels :: forall pxa pxb. (Pixel pxa, Pixel pxb)
+            => Traversal (Image pxa) (Image pxb) pxa pxb
+{-# INLINE imagePixels #-}
+imagePixels f Image { imageWidth = w, imageHeight = h, imageData = vec } =
+  freezeGenST w h <$> pixels
+  where
+    sourceComponentCount = componentCount (undefined :: pxa)
+    destComponentCount = componentCount (undefined :: pxb)
+
+    maxi = w * h * sourceComponentCount
+    pixels =
+      go (pure $ GenST $ M.new (w * h * destComponentCount)) 0 0
+
+    go act readIdx _ | readIdx >= maxi = act
+    go act readIdx writeIdx =
+      go newAct (readIdx + sourceComponentCount) (writeIdx + destComponentCount)
+      where
+        px = f (unsafePixelAt vec readIdx)
+        newAct = writePx writeIdx <$> act <*> px
+
+-- | Traversal providing the pixel position with it's value.
+-- The traversal in raster order, from lef to right, then top
+-- to bottom. The traversal match pixelMapXY in spirit.
+--
+-- Since 3.2.4
+imageIPixels :: forall pxa pxb. (Pixel pxa, Pixel pxb)
+             => Traversal (Image pxa) (Image pxb) (Int, Int, pxa) pxb
+{-# INLINE imageIPixels #-}
+imageIPixels f Image { imageWidth = w, imageHeight = h, imageData = vec } =
+  freezeGenST w h <$> pixels
+  where
+    sourceComponentCount = componentCount (undefined :: pxa)
+    destComponentCount = componentCount (undefined :: pxb)
+
+    pixels =
+      lineMapper (pure $ GenST $ M.new (w * h * destComponentCount)) 0 0 0
+
+    lineMapper act _ _ y | y >= h = act
+    lineMapper act readIdxLine writeIdxLine y =
+        go act readIdxLine writeIdxLine 0
+      where
+        go cact readIdx writeIdx x
+          | x >= w = lineMapper cact readIdx writeIdx $ y + 1
+          | otherwise = do
+             let px = f (x, y, unsafePixelAt vec readIdx)
+             go (writePx writeIdx <$> cact <*> px)
+                (readIdx + sourceComponentCount)
+                (writeIdx + destComponentCount)
+                (x + 1)
+
 -- | Just like `pixelMap` only the function takes the pixel coordinates as
 --   additional parameters.
 pixelMapXY :: forall a b. (Pixel a, Pixel b)
@@ -2051,7 +2123,7 @@ instance ColorSpaceConvertible PixelYCbCrK8 PixelRGB8 where
   convertPixel (PixelYCbCrK8 y cb cr _k) = PixelRGB8 (clamp r) (clamp g) (clamp b)
     where
       tof :: Word8 -> Float
-      tof v = fromIntegral v
+      tof = fromIntegral
 
       clamp :: Float -> Word8
       clamp = floor . max 0 . min 255
@@ -2066,7 +2138,7 @@ instance ColorSpaceConvertible PixelYCbCrK8 PixelCMYK8 where
   convertPixel (PixelYCbCrK8 y cb cr k) = PixelCMYK8 c m ye k
     where
       tof :: Word8 -> Float
-      tof v = fromIntegral v
+      tof = fromIntegral
 
       clamp :: Float -> Word8
       clamp = floor . max 0 . min 255
@@ -2466,3 +2538,5 @@ writePackedPixelAt img idx px =
     !converted =
         M.unsafeFromForeignPtr packedPtr s s2
 
+{-# ANN module "HLint: ignore Reduce duplication" #-}
+
